package demo;

import javax.ws.rs.GET;
import javax.ws.rs.Path;
import javax.ws.rs.Produces;
import javax.ws.rs.QueryParam;
import javax.ws.rs.WebApplicationException;
import javax.ws.rs.client.ClientBuilder;
import javax.ws.rs.core.MediaType;
import javax.ws.rs.core.Response;
import java.net.URI;
import java.util.List;
import java.util.concurrent.Callable;
import java.util.concurrent.Executors;
import java.util.concurrent.Future;
import java.util.stream.Collectors;

@Path("/")
public class AggregatorServices {

    /**
     * Endpoint: /anyOf?left=<endpoint>&right=<endpoint>
     *
     * Fetches the left and right resources in parallel, returning the first to
     * complete successfully, cancelling the other.
     */
    @GET
    @Path("anyOf")
    @Produces(MediaType.APPLICATION_JSON)
    public String anyOf(@QueryParam("left") String left,
                        @QueryParam("right") String right) throws Exception {

        if (left == null || right == null) {
            throw new WebApplicationException(Response.Status.BAD_REQUEST);
        }

        try (var executor = Executors.newVirtualThreadExecutor()) {
            Callable<String> task1 = () -> query(left);
            Callable<String> task2 = () -> query(right);

            // return the first to succeed, cancel the other
            return executor.invokeAny(List.of(task1, task2));
        }
    }

    /**
     * Endpoint: /allOf?left=<endpoint>&right=<endpoint>
     *
     * Fetches the left and right resources in parallel and combines the results. If
     * there is an error fetching a resource then it cancels fetching of the other.
     */
    @GET
    @Path("allOf")
    @Produces(MediaType.APPLICATION_JSON)
    public String allOf(@QueryParam("left") String left,
                        @QueryParam("right") String right) throws Exception {

        if (left == null || right == null) {
            throw new WebApplicationException(Response.Status.BAD_REQUEST);
        }

        try (var executor = Executors.newVirtualThreadExecutor()) {
            Callable<String> task1 = () -> query(left);
            Callable<String> task2 = () -> query(right);

            // if one fails, the other is cancelled
            return executor.invokeAll(List.of(task1, task2), true)
                    .stream()
                    .map(Future::join)
                    .collect(Collectors.joining(", ", "{ ", " }"));
        }
    }

    private String query(String endpoint) {
        URI uri = URI.create("http://localhost:8081").resolve(endpoint);
        return ClientBuilder.newClient()
                    .target(uri)
                    .request(MediaType.APPLICATION_JSON)
                    .get(String.class);
    }
}
